/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Sonatype, Nexus, Ext*/

// FIXME: Re-write as NX.define singleton=true

define('Nexus/clm', [
  'Nexus/config',
  'Nexus/messages',
  'Nexus/siesta'
], function () {

  Ext.namespace('Nexus.clm');

  /**
   * Base identifier for all plugins bits.
   *
   * @type {String}
   * @static
   */
  Nexus.clm.ID = 'nx-clm';

  /**
   * Title for various bits.
   *
   * @type {String}
   * @static
   */
  Nexus.clm.TITLE = 'CLM';

  /**
   * Title for dashboard.
   *
   * @type {String}
   * @static
   */
  Nexus.clm.TITLE_DASHBOARD = 'CLM Dashboard';

  /**
   * Title for the tab.
   * @type {string}
   */
  Nexus.clm.TAB_TITLE = "CLM";

  /**
   * ID for the tab.
   * @type {string}
   */
  Nexus.clm.TAB_ID = "clm";

  /**
   * The key for data collected in Staging form.
   * @type {string}
   */
  Nexus.clm.STAGING_KEY = 'CLMApplicationID';

  /**
   * The label to show in Staging form.
   * @type {string}
   */
  Nexus.clm.STAGING_LABEL = 'CLM Application';

  /**
   * Show a user-token transient message.
   *
   * @param message
   * @static
   */
  Nexus.clm.showMessage = function (message) {
    Nexus.messages.show(Nexus.clm.TITLE, message);
  };

  /**
   * Returns the path to a clm REST resource.
   *
   * @param path
   * @return {String}
   * @static
   */
  Nexus.clm.resourcePath = Nexus.siesta.basePath + '/clm';
  
  Nexus.clm.componentDetailContentResourcePath = Nexus.siesta.basePath + '/healthcheck/clm/assets/version-graph/rm/nexus';

  Nexus.clm.ruleTemplate = NX.create('Ext.XTemplate',
        '<div class="nx-clm-status {messageClass}">{message}</div>',
        '<div class="nx-staging-repository-ActivityView-messagePanel-head">',
          '<tpl if="violationCount &gt; 0">Violations were found in <b>{violationCount}</b> components.</tpl>',
          '<tpl if="violationCount == 0">No violations found.</tpl>',
        '</div>',
        '<div class="nx-clm-alerts">',
          '<tpl if="criticalCount &gt; 0">',
            '<div class="nx-clm-alert_box">',
              '<div class="nx-clm-chiclet critical">{criticalCount}</div>',
              '<div class="nx-clm-alert_critical_text">CRITICAL</div>',
            '</div>',
          '</tpl>',
          '<tpl if="severeCount &gt; 0">',
            '<div class="nx-clm-alert_box">',
             '<div class="nx-clm-chiclet severe">{severeCount}</div>',
              '<div class="nx-clm-alert_severe_text">SEVERE</div>',
            '</div>',
          '</tpl>',
          '<tpl if="moderateCount &gt; 0">',
            '<div class="nx-clm-alert_box">',
              '<div class="nx-clm-chiclet moderate">{moderateCount}</div>',
              '<div class="nx-clm-alert_moderate_text">MODERATE</div>',
            '</div>',
          '</tpl>',
          '<a href="{detailedReport}" target="_blank" class="view-report-button">View Full Report</a>',
        '</div>',
        '<br style="clear:both;"/>',
        {
          compiled : true
        }
  );

  Nexus.clm.chickletTemplate = NX.create('Ext.XTemplate',
        '<tpl if="criticalCount &gt; 0">',
          '<div class="nx-clm-chiclet-small critical">{criticalCount}</div>',
        '</tpl>',
        '<tpl if="severeCount &gt; 0">',
          '<div class="nx-clm-chiclet-small severe">{severeCount}</div>',
        '</tpl>',
        '<tpl if="moderateCount &gt; 0">',
          '<div class="nx-clm-chiclet-small moderate">{moderateCount}</div>',
        '</tpl>',
        '<tpl if="(criticalCount + severeCount + moderateCount) == 0">No violations</tpl>',
        '<tpl if="detailedReport">',
          '<span class="view-report-link-span">',
            '<a href="{detailedReport}" target="_blank" class="view-report-link">View Full Report</a>',
          '</span>',
        '</tpl>',
        {
          compiled : true
        }
  );

  Nexus.clm.stagingRule = function(event, detail) {
    if (detail.getPropertyValue('typeId') === 'clm') {
      var message, messageClass;

      var status = detail.getPropertyValue('x.appScan.status');
      if(status === undefined)
      {
        NX.log.error("Unable to determine status for CLM result:" + detail);
        return undefined;
      }

      switch (status) {
        case 'PASS':
          message = 'Validation was successful!';
          messageClass = 'nx-clm-status_success';
          break;
        case 'WARN':
          message = 'There are warnings.';
          messageClass = 'nx-clm-status_warn';
          break;
        case 'FAIL':
          message = 'Validation failed.';
          messageClass = 'nx-clm-status_fail';
          break;
        default :
          Nexus.clm.showMessage("Unrecognized CLM status: "+ details.getPropertyValue('x.appScan.status'));
      }
      var violationCount = detail.getPropertyValue('x.appScan.violationCount') || 0;
      var criticalCount = detail.getPropertyValue('x.appScan.threatLevelCount.CRITICAL') || 0;
      var severeCount = detail.getPropertyValue('x.appScan.threatLevelCount.SEVERE') || 0;
      var moderateCount = detail.getPropertyValue('x.appScan.threatLevelCount.MODERATE') || 0;

      return NX.create('Ext.XTemplate',
            '{[ this.render() ]}',
            {
              compiled : true,
              render : function() {
                return Nexus.clm.ruleTemplate.apply({
                  message: message,
                  messageClass: messageClass,
                  detailedReport : detail.getPropertyValue('x.appScan.url'),
                  violationCount : violationCount,
                  criticalCount : criticalCount,
                  severeCount : severeCount,
                  moderateCount : moderateCount
                });
              }
            });
    }

    return undefined;
  };
});
