/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/healthcheck/HealthCheckArtifactDetailsContrib', [ 'Nexus/healthcheck/HealthCheckUtil', 'Nexus/healthcheck/HealthCheckProUtil', 'Nexus/clm/ComponentDetailView' ], function() {

  HealthCheckArtifactDetailsContrib = function() {
      function getInsight(panel) {
        if (panel.items.getCount() === 2) {
          return panel.items.itemAt(1).iframe.dom.contentWindow.Insight;  
        }
      }
      
      var me = this;

      me.getLoadingHtml = function() {
          return '<div style="background:#ececec; width: 100%; height: 100%; padding: 20px;font-size: 14px;font-family: Arial, Helvetica, sans-serif">Loading...</div>';
      };

      me.getFileUrl = function() {
          return Sonatype.HealthCheckUtil.getBase() + '/healthCheckFileDetail/' + Sonatype.utils.base64.encode(me.data.resourceURI) + '/index.html';
      };

      me.getGavUrl = function() {
          return Sonatype.HealthCheckUtil.getBase() + '/healthCheckArtifactDetail/' + me.data.groupId + '/' + me.data.artifactId + '/' + me.data.version + '/index.html';
      };
      me.toTabId = function(appId, groupId, artifactId, version, hash) {
        return 'compDetail' + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(appId)
                + Sonatype.view.HISTORY_DELIMITER + encodeURIComponent(groupId) + Sonatype.view.HISTORY_DELIMITER
                + encodeURIComponent(artifactId) + Sonatype.view.HISTORY_DELIMITER
                + encodeURIComponent(version) + Sonatype.view.HISTORY_DELIMITER
                + (hash ? encodeURIComponent(hash) : '');
      };
      
      me.registerClmViewDetailsListener = function(panel) {
         getInsight(panel).registerViewDetailsListener(
                function(appId, groupId, artifactId, version, classifier, extension, hash, matchState,
                        proprietary) {

                  Sonatype.view.mainTabPanel.addOrShowTab(
                          me.toTabId(appId, groupId, artifactId, version, hash), Nexus.clm.ComponentDetailView,
                          {
                            title: 'CLM Detail',
                            appId: appId,
                            groupId: groupId,
                            artifactId: artifactId,
                            version: version,
                            hash: hash
                          });
                });
      };

      me.getGav = function() {
        if (me.data && me.data.groupId) {
          return {
            groupId: me.data.groupId,
            artifactId: me.data.artifactId,
            version: me.data.version
          };
        } else if (me.data && me.data.resourceURI) {
          var segments = me.data.resourceURI.split('/'), groupId = '';

          for ( var i = 0; i < segments.length; i++) {
            if (segments[i] === 'content') {
              segments.splice(0, i + 1);
              break;
            }
          }

          if (segments.length < 4) {
            return null;
          }

          for ( var i = 0; i < segments.length - 3; i++) {
            groupId += '.' + segments[i];
          }
          groupId = groupId.substring(1);

          return {
            groupId: groupId,
            artifactId: segments[segments.length - 3],
            version: segments[segments.length - 2]
          }
        }
      };
      
      me.setAuthFailureHandler = function(panel) {
        getInsight(panel).setAuthFailureHandler(function () {
          Sonatype.repoServer.RepoServer.loginHandler();
        });
      };
      
      me.isClmLoaded = function(panel) {
        return getInsight(panel);
      };

      me.setClmGav = function(panel) {
        var gav = me.getGav();
        if (gav) {
          getInsight(panel).setGav(gav);
        } else {
          getInsight(panel).clearGav();
        }
      };

      me.buildLoadingPanel = function() {
          return new Ext.Panel({
              width : '100%',
              height : '100%',
              border : true,
              collapsible : false,
              collapsed : false,
              html : me.getLoadingHtml()
          });
      };

      me.buildDetailPanel = function(config) {
          var config = config || {};
          var defaultConfig = {
              width : '100%',
              height : '100%',
              border : true,
              collapsible : false,
              collapsed : false,
              layout : 'card',
              items : [ me.buildLoadingPanel() ]
          };

          Ext.applyIf(config, defaultConfig);
          return new Ext.Panel(config);
      };

      me.rebuildIframe = function(panel, url) {
          if (me.isClmLoaded(panel)) {
              // set GAV
              me.setClmGav(panel);
          } else {
              //item 0 is the loading panel, don't ever remove that one
              panel.getLayout().setActiveItem(0);

              var old = panel.items.get(1);
              if (old) {
                  panel.remove(old);
                  old.destroy();
              }

              panel.add({
                xtype: 'iframepanel',
                border: false,
                disableMessaging: false,
                defaultSrc: url,
                listeners: {
                  domready: function() {
                    panel.getLayout().setActiveItem(1);
                    if (me.isClmLoaded(panel)) {
                      me.setClmGav(panel);
                      me.registerClmViewDetailsListener(panel);
                      me.setAuthFailureHandler(panel);
                    }
                  }
                }
              });

              panel.doLayout();
          }
      };

      me.viewDetails = function() {
          if (me.artifactPanel && me.data && me.data.groupId && me.data.artifactId && me.data.version) {
              me.rebuildIframe(me.artifactPanel, me.getGavUrl());
          } else if (me.filePanel && me.data && me.data.resourceURI) {
              me.rebuildIframe(me.filePanel, me.getFileUrl());
          }
      };

      me.allowView = function() {
          if (!Sonatype.HealthCheckProUtil.hasAlertReadPerm() || !Sonatype.HealthCheckProUtil.hasPaidProLicense()) {
              return false;
          }

          return true;
      };

      me.showListener = function() {
          me.shown = true;

          if (!me.dataDisplayed && me.data) {
              me.dataDisplayed = true;
              me.viewDetails();
          }
      };

      me.hideListener = function() {
          me.shown = false;
      };

      Sonatype.Events.addListener('fileContainerInit', function(items) {
          if (me.allowView()) {
              me.filePanel = me.buildDetailPanel({
                  name : 'healthCheckFileDetailPanel',
                  tabTitle : 'Component Info',
                  preferredIndex : 80,
                  listeners : {
                      show : me.showListener,
                      hide : me.hideListener
                  }
              });
              items.push(me.filePanel);
          }
      });

      Sonatype.Events.addListener('fileContainerUpdate', function(artifactContainer, data) {
          if (me.allowView()) {
              me.data = data;
              me.dataDisplayed = false;

              if (me.shown) {
                  me.dataDisplayed = true;
                  me.viewDetails();
              }
          }
      });

      Sonatype.Events.addListener('artifactContainerInit', function(items) {
          if (me.allowView()) {
              me.artifactPanel = me.buildDetailPanel({
                  name : 'healthCheckArtifactDetailPanel',
                  tabTitle : 'Component Info',
                  preferredIndex : 80,
                  listeners : {
                      show : me.showListener,
                      hide : me.hideListener
                  }
              });
              items.push(me.artifactPanel);
          }
      });

      Sonatype.Events.addListener('artifactContainerUpdate', function(artifactContainer, data) {
          if (me.allowView()) {
              me.data = data;
              me.dataDisplayed = false;

              if (me.shown) {
                  me.dataDisplayed = true;
                  me.viewDetails();
              }
          }
      });

      Sonatype.Events.addListener('historyChanged', function(token) {
        if (token) {
          var toks = token.split(Sonatype.view.HISTORY_DELIMITER);
          if (toks[0] == 'compDetail') {
            if (Sonatype.user.curr.isLoggedIn) {
              if (me.allowView()) {
                var appId = decodeURIComponent(toks[1]);
                var groupId = decodeURIComponent(toks[2]);
                var artifactId = decodeURIComponent(toks[3]);
                var version = decodeURIComponent(toks[4]);
                var hash = decodeURIComponent(toks[5]);
                var id = me.toTabId(appId, groupId, artifactId, version, hash);
  
                Sonatype.view.mainTabPanel.addOrShowTab(id, Nexus.clm.ComponentDetailView, {
                  title: 'CLM Detail',
                  appId: appId,
                  groupId: groupId,
                  artifactId: artifactId,
                  version: version,
                  hash: hash
                });
              }
            } else {
              Sonatype.view.afterLoginToken = token;
              Sonatype.repoServer.RepoServer.loginHandler();
            }
            // was handled by this event listener so return false to stop others
            return false;
          }
        }
      });
    };

  Ext.onReady(function() {
      Sonatype.HealthCheckArtifactDetailsContrib = new HealthCheckArtifactDetailsContrib();
  });
});