/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, NX, Sonatype, Nexus*/
(function() {
  var sp = Sonatype.lib.Permissions, may, mayDelete, removeMessage;

  may = function(term) {
    return sp.checkPermission('nexus:ssl:truststore', term);
  };

  mayDelete = function() {
    return may(sp.DELETE);
  };

  removeMessage = new Ext.XTemplate(
        'Remove the following certificates?<br/>',
        '(You need to restart Nexus to make sure certificates are not cached anymore)<br/><br/>',
        // XTemplate is behaving weird for an array of records,
        // `<tpl for=".">{data.certificate.subjectCommonName}</tpl>` did not work
        '<tpl for=".">', '<tpl for="data">',
        '{subjectCommonName}<br/>',
        '</tpl>', '</tpl>', {
          disableFormats : true,
          compiled : true
        }
  );

  NX.define('Nexus.ssl.configuration.CertificateManagerGrid', {

    extend : 'Ext.grid.EditorGridPanel',
    requireSuper : false,
    requirejs : ['Nexus/config'],
    requires : ['Nexus.ssl.Icons', 'Nexus.ssl.CertificateStore', 'Nexus.ssl.certificate.AddByServerFetch', 'Nexus.ssl.certificate.AddByPEM'],

    xtype : 'certificate-manager',

    mixins : ['Nexus.LogAwareMixin'],

    select : function(id) {
      var self = this, record;

      if (id) {
        record = self.store.query('id', id).get(0);
        if (record) {
          if (record.id !== self.selectedId) {
            // may have to wait for viewReady, or we are trying to select on an empty grid...
            if (self.getView().getRows().length === 0) {
              self.on('viewready', function() {
                self.getSelectionModel().selectRecords([record]);
                self.selectedId = id;
              }, self, {single : true});
            } else {
              self.getSelectionModel().selectRecords([record], true);
              self.selectedId = id;
              return true;
            }
          }
        }
      }

      return false;
    },
    initComponent : function() {
      var self = this,
          icons = Nexus.ssl.Icons;

      self.border = true;

      self.selModel = new Ext.grid.RowSelectionModel({
        listeners : {
          selectionchange : function(selModel) {
            self.getTopToolbar().get('remove').setDisabled(!selModel.hasSelection());
          },
          rowselect : function(selModel, idx, record) {
            self.fireEvent('certSelected', record);
          },
          rowdeselect : function(selModel, idx, record) {
            self.fireEvent('certDeselected', record);
          }
        }
      });

      self.store = Nexus.ssl.CertificateStore;

      self.tbar = [
        {
          iconCls : icons.get('refresh').cls,
          text : 'Refresh',
          handler : function() {
            self.store.reload();
          }
        },
        {
          iconCls : icons.get('add_certificate').cls,
          text : 'Add...',
          menu : [
            {
              text : 'Load from server',
              handler : function() {
                Ext.create({ xtype : 'ssl-add-by-server' }).asWindow('Load certificate from server').show();
              }
            },
            {
              text : 'Paste PEM',
              handler : function() {
                Ext.create({ xtype : 'ssl-add-by-pem' }).asWindow('Paste certficate as PEM').show();
              }
            }
          ]
        },
        {
          text : 'Delete',
          itemId : 'remove',
          iconCls : icons.get('delete_certificate').cls,
          disabled : true,
          handler : function() {
            var records = self.getSelectionModel().getSelections();
            Sonatype.utils.defaultToNo();

            Sonatype.MessageBox.show({
              animEl : this.getEl(),
              title : 'Delete selected certificates?',
              msg : removeMessage.apply(records),
              buttons : Sonatype.MessageBox.YESNO,
              scope : this,
              icon : Sonatype.MessageBox.WARNING,
              fn : function(btnName) {
                if (btnName === 'yes' || btnName === 'ok') {
                  Ext.each(records, function(record) {
                    self.store.remove(record);
                  });

                  self.store.reload();
                  self.fireEvent('certsRemoved', records);
                }
              }
            });
          },
          hidden : !mayDelete()
        }
      ];

      self.autoExpandColumn = 'fingerprint';

      self.colModel = new Ext.grid.ColumnModel({
        defaultSortable : true,
        columns : [
          // icon
          {
            width: 30,
            resizable: false,
            sortable: false,
            fixed: true,
            hideable: false,
            menuDisabled: true,
            renderer: function (value, metaData, record) {
                return icons.get('certificate').img;
            }
          },
          {header : 'Name', dataIndex : 'subjectCommonName', width : 100},
          {header : 'Issued To', dataIndex : 'subjectOrganization', width : 100},
          {header : 'Issued By', dataIndex : 'issuerOrganization', width : 100},
          {id : 'fingerprint', header : 'Fingerprint', dataIndex : 'fingerprint'}
        ]
      });

      self.on('rowclick', function(grid, idx) {
        self.selectedId = self.store.getAt(idx).id;
      });

      self.constructor.superclass.initComponent.apply(self, arguments);

      self.on('render', function() {
        self.mask = new Ext.LoadMask(self.getGridEl(), {
          msg : "Loading certificates...",
          store : self.store
        });
        self.store.load();
      });

//      need to disable the mask because it would try to mask already closed grid
      self.on('beforedestroy', function() {
        self.mask.disable();
      });
    }
  });
}());
