/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/

//
// FIXME: This is now only needed by staging upload UI, but really that UI needs to be updated anyways to handle async + activities for this,
// FIXME: so likely this will completely go away once that is updated.
//

define('Nexus/staging/Rules', function() {

    Ext.namespace('Nexus.staging');
    (function() {

      /**
       * Collection of methods that defines behavior.
       * @param methods An object holding methods that will be applied to an enhanced objects' prototype.
       * @constructor
       */
      var Trait = function(methods) {
        /**
         * @static
         * @param obj The object to enhance with this Trait's method collection.
         */
        this.treat = function(obj) {
          if (obj.prototype) {
            obj = obj.prototype;
          }
          Ext.apply(obj, methods);
        };
      };

      Nexus.staging.Rules = new Trait({
        generateHtmlFromRulesetFailure : function(responseText) {
          var
                json = Ext.decode(responseText),
                html = "<p><h3>Staging ruleset evaluation on repository '" + json.repositoryName + "' has failed.</h3>";

          Ext.each(json.failures, function(item) {
            html += "<br/><p><h3>" + item.ruleName + "</h3></p>";
            Ext.each(item.messages, function(msg) {
              html += "&nbsp;&nbsp;&nbsp;&nbsp;-" + msg.replace(/^([^:]*):/, '<b>$1:</b>') + "<br/>";
            });
          });

          html += "</p>";

          return html;
        },
        showRulesetEvaluationReport : function(responseText) {
          var reportWindow = new Ext.Window({
            id : 'ruleset-evaluation-report',
            title : 'Ruleset Evaluation Report',
            layout : 'border',
            width : 600,
            height : 300,
            buttonAlign : 'center',
            items : [
              {
                xtype : 'panel',
                autoScroll : true,
                html : this.generateHtmlFromRulesetFailure(responseText),
                region : 'center'
              }
            ],
            buttons : [
              {
                text : 'OK',
                scope : this,
                handler : function() {
                  Ext.getCmp('ruleset-evaluation-report').close();
                }
              }
            ]
          });
          reportWindow.show();
        }
      });
    })();

})

