/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repositories master/detail view.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.RepositoriesView', {
    extend: 'Ext.Panel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.Icons',
        'Nexus.staging.repository.RepositoriesGrid',
        'Nexus.staging.repository.RepositoryView',
        'Nexus.staging.repository.RepositoriesMediator'
    ],

    /**
     * @override
     */
    initComponent: function () {
        var self = this,
            icons = Nexus.staging.Icons;

        // force all datastores to refresh when the main staging repositories view loads
        Nexus.staging.repository.RepositoriesMediator.refreshHandler();

        self.masterPanel = NX.create('Nexus.staging.repository.RepositoriesGrid', {
            region: 'center'
        });

        self.emptySelectionPanel = NX.create('Ext.Panel', {
            cls: 'nx-staging-repository-RepositoriesView-messagePanel',
            title: 'Empty Selection',
            iconCls: icons.get('selectionEmpty').cls,
            html: '<span class="nx-staging-repository-RepositoriesView-messagePanel-text">Please select a staging repository</span>'
        });

        self.multiSelectionPanel = NX.create('Ext.Panel', {
            cls: 'nx-staging-repository-RepositoriesView-messagePanel',
            title: 'Multiple Selection',
            iconCls: icons.get('selectionMulti').cls,
            html: '<span class="nx-staging-repository-RepositoriesView-messagePanel-text">Multiple staging repositories are selected</span>'
        });

        self.repoView = NX.create('Nexus.staging.repository.RepositoryView');

        self.detailPanel = NX.create('Ext.Panel', {
            region: 'south',
            minHeight: 25, // height of panel title
            split: true,
            autoDestroy: false,

            layout: 'card',
            defaults: {
                border: false
            },
            items: [
                self.emptySelectionPanel, // 0
                self.multiSelectionPanel, // 1
                self.repoView             // 2
            ],
            activeItem: 0,

            listeners: {
                // resize detail panel height to 50% of parent on first render
                afterrender: {
                    single: true,
                    fn: function() {
                        self.detailPanel.setHeight(self.getHeight() / 2);
                    }
                }
            }
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-RepositoriesView',
            layout: 'border',
            items: [
                self.masterPanel,
                self.detailPanel
            ]
        });

        self.masterPanel.getSelectionModel().on('selectionchange', self.selectionChanged, self);

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * Update detail panel when grid selection changes.
     *
     * @param sm    grid selection model
     *
     * @private
     */
    selectionChanged: function(sm) {
        var self = this,
            cardLayout = self.detailPanel.getLayout(),
            selections = sm.getSelections();

        self.logDebug('Selection changed:', selections.length);

        if (selections.length === 0) {
            cardLayout.setActiveItem(0); // empty-selection
        }
        else if (selections.length === 1) {
            cardLayout.setActiveItem(2); // single-selection

            // update after layout changed, so that tabs can properly [un]hide
            self.repoView.updateRecord(selections[0].data);
        }
        else {
            cardLayout.setActiveItem(1); // multi-selection
        }
    }

}, function () {
    var type = this,
        sp = Sonatype.lib.Permissions;

    NX.log.debug('Adding global view: ' + type.$className);

    // install panel into main NX navigation
    Sonatype.Events.on('nexusNavigationInit', function (panel) {
        panel.add({
            enabled: sp.checkPermission('nexus:stagingprofilerepos', sp.READ),
            sectionId: 'st-enterprise-staging',
            title: 'Staging Repositories',
            tabId: 'stagingRepositories',
            tabCode: type
        });

        NX.log.debug('Registered global view: ' + type.$className);
    });
})