/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Sonatype.repoServer.ArtifactResolutionMenu', {
  requires : [
    'Sonatype.repoServer.ArtifactProcurementRulePickerDialog',
    'Sonatype.repoServer.ArtifactProcurementRulePopup'
  ],
  constructor : function(menu, node) {
    this.menu = menu;
    this.node = node;
    this.selectedLicense = "";

    if (Sonatype.lib.Permissions.checkPermission('nexus:procurementresolution', Sonatype.lib.Permissions.CREATE)) {

      this.approveIcon = Sonatype.config.resourcePath + '/static/images/approve.png';
      this.denyIcon = Sonatype.config.resourcePath + '/static/images/disapprove.png';

      this.approveActions = [];
      this.denyActions = [];
      if (node.isRoot) {
        this.addRootActions(node);
      }
      else if (node.attributes.type == 'G') {
        var groupId = node.attributes.path;
        this.groupId = groupId.substring(1, groupId.length - 1).replace(/\//g, '.');
        this.addGroupActions(node);
      }
      else if (node.attributes.type == 'A') {
        this.groupId = node.attributes.groupId;
        this.artifactId = node.attributes.artifactId;
        this.addArtifactActions(node);
      }
      else if (node.attributes.type == 'V' || node.attributes.type == 'artifact') {
        this.groupId = node.attributes.groupId;
        this.artifactId = node.attributes.artifactId;
        this.version = node.attributes.version;
        this.addVersionActions(node);
      }
    }

    if ((node.attributes.appliedArtifactResolutionIds && node.attributes.appliedArtifactResolutionIds.length)
          || node.isRoot) {
      menu.addSeparator();
      menu.add({
        text : 'Applied Rules...',
        handler : function(button, e) {
          new Sonatype.repoServer.ArtifactProcurementRulePopup(node);
        }
      });
    }
  },

  addAction : function(text, mask, node) {
    var title = text + ' (' + mask + ')';
    this.menu.add({
      text : title,
      scope : this,
      handler : this.showRuleDialog.createDelegate(this, [mask, node])
    });
  },

  addRootActions : function(node) {
    this.addMenuHeader();
    this.addAction('All groups, artifacts and versions', '*:*:*', node);
    this.addAction('A User-Entered GAV', Nexus.procurement.configuration.FREEFORM_GAV, node);
  },

  addArtifactActions : function(node) {
    this.addGroupActions(node);
    this.addSeparator();
    this.addAction('Everything with this artifact id', '*:' + this.artifactId + ':*', node);
    this.addAction('Any artifact starting with this prefix', '*:' + this.artifactId + '*:*', node);
    this.addAction('All versions of this artifact', this.groupId + ':' + this.artifactId + ':*', node);
  },

  addMenuHeader : function() {
    var ti = new Ext.menu.TextItem();
    ti.text = 'Apply a Rule To:';
    ti.style = 'text-align:center;padding-left:27px;';
    this.menu.add(ti);
  },

  addGroupActions : function(node) {
    this.addMenuHeader();
    this.addAction('Everything in this group', this.groupId + ':*:*', node);
    this.addAction('Everything in this group and its sub-groups', this.groupId + '.*:*:*', node);
    this.addAction('Any group starting with this prefix', this.groupId + '*:*:*', node);
  },

  addVersionActions : function(node) {
    this.addArtifactActions(node);
    this.addSeparator();
    this.addAction('Exactly this artifact', this.groupId + ':' + this.artifactId + ':' + this.version, node);
  },

  addSeparator : function() {
    this.menu.addSeparator();
  },

  showRuleDialog : function(mask, node) {
    new Sonatype.repoServer.ArtifactProcurementRulePickerDialog(mask, node);
  },

  approve : function(mask) {
    this.procure('APPROVED', mask);
  },

  deny : function(mask) {
    this.procure('DISAPPROVED', mask);
  },

  procure : function(resolution, mask) {
    var repoRec = this.node.ownerTree.payload;
    var repoName = repoRec.data.name;
    var gav = mask.split(':');

    Ext.Ajax.request({
      method : 'POST',
      jsonData : {
        data : {
          artifactCoordinate : {
            groupId : gav[0],
            artifactId : gav[1],
            version : gav[2]
          },
          resolution : resolution
        }
      },
      url : Nexus.procurement.configuration.PROCUREMENT_RESOLUTION_SERVICE_PATH + repoRec.data.id,
      scope : this,
      callback : function(options, success, response) {
        if (success) {
          this.node.ownerTree.refreshNode(this.node);
        }
        else {
          Sonatype.utils.connectionError(response, 'Artifact resolution failed in ' + repoName + '.');
        }
      }
    });
  }
}, function() {
  Sonatype.Events.addListener('procurementNodeMenuInit', function(menu, node) {
    new Sonatype.repoServer.ArtifactResolutionMenu(menu, node);
  });
});
