/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus, define*/

define('Sonatype/repoServer/StagingUploadPanel', ['Nexus/staging/Rules'], function() {

    Sonatype.repoServer.StagingUploadPanel = function(config) {
      var config = config || {};

      var availableModes =[];
      if(Sonatype.lib.Permissions.checkPermission('nexus:stagingupload', Sonatype.lib.Permissions.CREATE) )
      {
        availableModes.push(['pom', 'Artifact(s) with a POM']);
        availableModes.push(['gav', 'Artifact(s) with GAV']);
      }

      if (Sonatype.lib.Permissions.checkPermission('nexus:stagingbundleupload', Sonatype.lib.Permissions.CREATE))
      {
        availableModes.push(['bundle', 'Artifact Bundle']);
      }

      var defaultConfig = {
        id : 'staging-upload',
        uploadModeTitle : 'Select Staging Upload Mode',
        uploadModeLabel : 'Upload Mode',
        uploadModeStore : new Ext.data.SimpleStore({
              fields : ['value', 'display'],
              data : availableModes
            }),
        uploadModeHt : 'The mode of staging upload, which depends on how you want to deploy your artifacts.',
        uploadModeExplanation : '<b>Artifact(s) with a POM:</b> GAV will be defined from a POM file.<br/>' + '<b>Artifact(s) with GAV:</b> GAV needs to be manually defined.<br/>'
            + '<b>Artifact Bundle:</b> A bundle file produced by <a href="http://maven.apache.org/plugins/maven-repository-plugin/">Maven Repository Plugin</a>, which should contain the POM.',
        extraItems : {
          xtype : 'textarea',
          fieldLabel : 'Description',
          helpText : 'A detailed description of the staged repository',
          anchor : Sonatype.view.FIELD_OFFSET,
          name : 'description',
          allowBlank : false,
          itemCls : 'required-field'
        }
      };
      Ext.apply(this, config, defaultConfig);

      // need to set invalid repo id, otherwise parent panel will fail
      // also setting to release policy, so validation will work properly
      this.payload = {
        id : 'invalid-repo-id',
        data : {
          repoPolicy : 'release'
        }
      };

      Sonatype.repoServer.StagingUploadPanel.superclass.constructor.call(this, {});

      this.on('beforerender', this.beforeRenderHandler, this);
    };

    Ext.extend(Sonatype.repoServer.StagingUploadPanel, Sonatype.repoServer.ArtifactUploadPanel, {
          createUploadForm : function(treePanel, fileInput, classifier, extension, lastItem) {
            var pomMode = this.find('name', 'gavDefinition')[0].getValue() == 'pom', tmpFormChildren = [];

            if (this.gavResponse)
            {
              pomMode = false;
              this.form.findField('g').setValue(this.gavResponse.groupId);
              this.form.findField('a').setValue(this.gavResponse.artifactId);
              this.form.findField('v').setValue(this.gavResponse.version);
              this.form.findField('p').setValue(this.gavResponse.packaging);

              tmpFormChildren.push({
                tag : 'input',
                type : 'hidden',
                name : 'r',
                value : this.gavResponse.repositoryId
              });
            }

            if (pomMode) {
              tmpFormChildren = tmpFormChildren.concat([
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'hasPom',
                  value : 'true'
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'c',
                  value : classifier
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'e',
                  value : extension
                }
              ]);
            } else {
              tmpFormChildren = tmpFormChildren.concat([
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'g',
                  value : this.form.findField('g').getValue()
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'a',
                  value : this.form.findField('a').getValue()
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'v',
                  value : this.form.findField('v').getValue()
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'p',
                  value : this.form.findField('p').getValue()
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'c',
                  value : classifier
                },
                {
                  tag : 'input',
                  type : 'hidden',
                  name : 'e',
                  value : extension
                }
              ]);
            }

            tmpFormChildren.push({
              tag : 'input',
              type : 'hidden',
              name : 'desc',
              value : this.form.findField('description').getValue()
            });

            var tmpForm = Ext.getBody().createChild({
                  tag : 'form',
                  cls : 'x-hidden',
                  id : Ext.id(),
                  children : tmpFormChildren
                });

            if (pomMode)
            {
              this.pomInput.appendTo(tmpForm);
            }
            if (fileInput)
            {
              fileInput.appendTo(tmpForm);
            }

            Ext.Ajax.request({
                  url : Sonatype.config.servicePath + '/staging/upload',
                  form : tmpForm,
                  isUpload : true,
                  cbPassThru : {
                    description : this.form.findField('description').getValue(),
                    treePanel : treePanel
                  },
                  callback : function(options, success, response) {
                    tmpForm.remove();

                    // This is a hack to get around the fact that upload submit
                    // always returns
                    // success = true
                    if (response.responseText.toLowerCase().indexOf('<error>') == -1)
                    {
                      this.gavResponse = Ext.decode(response.responseText);
                      if (lastItem)
                      {
                        this.finishRepository(options.cbPassThru.description);
                        this.gavResponse = null;
                      }
                      else
                      {
                        this.currentChildNode++;
                        var treePanel = options.cbPassThru.treePanel;
                        if (this.currentChildNode < treePanel.root.childNodes.length)
                        {
                          this.createUploadForm(treePanel, treePanel.root.childNodes[this.currentChildNode].attributes.payload.fileInput, treePanel.root.childNodes[this.currentChildNode].attributes.payload.classifier,
                              treePanel.root.childNodes[this.currentChildNode].attributes.payload.extension, this.currentChildNode == treePanel.root.childNodes.length - 1);
                        }
                      }
                    }
                    else
                    {
                      var s = 'Artifact upload failed.<br />';
                      var r = response.responseText;
                      var n1 = r.toLowerCase().indexOf('<error>') + 7;
                      var n2 = r.toLowerCase().indexOf('</error>');
                      if (n2 > n1)
                      {
                        s += r.substring(n1, n2);
                      }
                      else
                      {
                        s += 'Check Nexus logs for more information.';
                      }
                      Sonatype.MessageBox.show({
                            title : 'Upload Failed',
                            msg : s,
                            buttons : Sonatype.MessageBox.OK,
                            icon : Sonatype.MessageBox.ERROR
                          });
                    }
                  },
                  scope : this
                });
          },
          getJsonFromHtmlResponse : function(response) {
            var r = response.responseText;
            var n1 = r.toLowerCase().indexOf('<pre>') + 5;
            var n2 = r.toLowerCase().indexOf('</pre>');
            var json = '';
            if (n2 > n1)
            {
              json = r.substring(n1, n2);
            }
            else
            {
              json = r;
            }

            return Ext.util.JSON.decode(json);
          },
          finishRepository : function(description) {
            var profileId = this.gavResponse.profileId;
            var repositoryId = this.gavResponse.repositoryId;

            Ext.Ajax.request({
                  scope : this,
                  method : 'POST',
                  options : {
                    dontForceLogout : true,
                    profileId : profileId,
                    repositoryId : repositoryId
                  },
                  callback : function(options, isSuccess, response) {
                    if (!isSuccess)
                    {
                      // async
                      this.dropRepository(options.options.profileId, options.options.repositoryId);

                      if (response.responseText.indexOf('failures') !== -1)
                      {
                        Sonatype.MessageBox.hide();
                        this.showRulesetEvaluationReport(response.responseText);
                      }
                      else
                      {
                        Sonatype.utils.connectionError(response, 'The server did not close the staging repository, it will be dropped.', false, options.options);
                      }
                    }
                    else
                    {
                      Sonatype.MessageBox.show({
                            title : 'Upload Complete',
                            msg : 'Artifact upload finished successfully',
                            buttons : Sonatype.MessageBox.OK,
                            icon : Sonatype.MessageBox.INFO
                          });
                      this.resetFields();
                    }
                  },
                  jsonData : {
                    data : {
                      stagedRepositoryId : repositoryId,
                      description : description
                    }
                  },
                  url : Sonatype.config.servicePath + '/staging/profiles/' + profileId + '/finish'
                });
          },
          dropRepository : function(profileId, repositoryId) {
            Ext.Ajax.request({
                  scope : this,
                  method : 'POST',
                  options : {
                    dontForceLogout : true
                  },
                  url : Sonatype.config.servicePath + '/staging/profiles/' + profileId + '/drop',
                  jsonData : {
                    data : {
                      stagedRepositoryId : repositoryId
                    }
                  },
                  callback : function(options, isSuccess, response) {
                    if (!isSuccess)
                    {
                      Sonatype.utils.connectionError(response, 'The server did not close the staging repository, and failed to drop it.', false, options.options);
                    }
                  }
                });
          },
          gavDefinitionSelectHandler : function(combo, record, index) {
            var gavDefinitionPanel = this.findById('gav-definition-card-panel');
            var artifactsUploadFieldset = this.findById('artifacts-upload-fieldset');
            // First disable all the items currently on screen, so they wont be
            // validated/submitted etc
            gavDefinitionPanel.getLayout().activeItem.items.each(function(item) {
                  if (item.xtype != 'browsebutton')
                  {
                    item.disable();
                  }
                });
            this.clearInvalid();
            // Then find the proper card to activate (based upon the selected
            // schedule type)
            if (record.data.value == 'pom')
            {
              gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(1));
              artifactsUploadFieldset.setVisible(true);
            }
            else if (record.data.value == 'gav')
            {
              gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(2));
              artifactsUploadFieldset.setVisible(true);
            }
            else if (record.data.value == 'bundle')
            {
              gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(3));
              artifactsUploadFieldset.setVisible(false);
            }
            else
            {
              gavDefinitionPanel.getLayout().setActiveItem(gavDefinitionPanel.items.itemAt(0));
            }
            // Then enable the fields in that card
            gavDefinitionPanel.getLayout().activeItem.items.each(function(item) {
                  if (item.xtype != 'browsebutton')
                  {
                    item.enable();
                    if (item.readOnly)
                    {
                      item.getEl().dom.readOnly = true;
                    }
                  }
                  else
                  {
                    item.setClipSize();
                  }
                });
            this.doLayout();
          },
          beforeRenderHandler : function(formPanel) {
            var bundleUploadEnabled = Sonatype.lib.Permissions.checkPermission('nexus:stagingbundleupload', Sonatype.lib.Permissions.CREATE);
            var gavDefinitionPanel = this.findById('gav-definition-card-panel');
            gavDefinitionPanel.add({
                  xtype : 'fieldset',
                  border : false,
                  checkboxToggle : false,
                  collapsible : false,
                  style : 'margin-right:7px;',
                  autoHeight : true,
                  buttonAlign : 'center',
                  items : [{
                        hideLabel : true,
                        name : 'upload-bundle-button',
                        xtype : 'browsebutton',
                        text : 'Select Bundle to Upload...',
                        style : 'margin-bottom: 5px;',
                        disabled : !bundleUploadEnabled,
                        uploadPanel : this,
                        handler : function(b) {
                          b.uploadPanel.bundleInput = b.detachInputFile();
                          var bundleName = b.uploadPanel.bundleInput.getValue();
                          b.uploadPanel.updateBundleFileName(b.uploadPanel, bundleName);
                        }
                      }, {
                        xtype : 'textfield',
                        fieldLabel : 'Bundle Filename',
                        name : 'bundle-name-field',
                        anchor : Sonatype.view.FIELD_OFFSET,
                        readOnly : true,
                        allowBlank : false,
                        itemCls : 'required-field',
                        disabled : true
                      }],
                  buttons : [{
                        xtype : 'button',
                        id : 'upload-bundle-button',
                        text : 'Upload Bundle',
                        handler : this.uploadBundleHandler,
                        disabled : !bundleUploadEnabled,
                        scope : this
                      }, {
                        xtype : 'button',
                        id : 'reset-bundle-button',
                        text : 'Reset',
                        handler : this.resetBundleHandler,
                        disabled : !bundleUploadEnabled,
                        scope : this
                      }]
                });
          },
          updateBundleFileName : function(uploadPanel, bundleName) {
            var bundleNameField = uploadPanel.find('name', 'bundle-name-field')[0];
            if (bundleName)
            {
              bundleNameField.setValue(bundleName);
            }
          },
          resetBundleHandler : function() {
            var bundleNameField = this.find('name', 'bundle-name-field')[0];
            bundleNameField.reset();
            this.bundleInput = null;
          },
          uploadBundleHandler : function() {
            if (!this.bundleInput)
            {
              Sonatype.MessageBox.show({
                    title : 'No Bundle Selected',
                    msg : 'You must select a bundle produced by the Maven Repository Plugin.',
                    buttons : Sonatype.MessageBox.OK,
                    icon : Sonatype.MessageBox.ERROR
                  });
            }
            else
            {
              Sonatype.MessageBox.wait('Uploading ...');
              var bundleForm = Ext.getBody().createChild({
                    tag : 'form',
                    cls : 'x-hidden',
                    id : Ext.id(),
                    children : []
                  });
              this.bundleInput.appendTo(bundleForm);
              Ext.Ajax.request({
                    url : Sonatype.config.servicePath + '/staging/bundle_upload',
                    form : bundleForm,
                    isUpload : true,
                    scope : this,
                    callback : function(options, success, response) {
                      // file uploads have to be forced to always succeed for IE, so we have
                      // to parse the response
                      if (response.responseText.toLowerCase().indexOf('<error>') == -1)
                      {
                        if (Ext.isEmpty(response.responseText))
                        {
                          Sonatype.MessageBox.show({
                                title : 'Upload Failed',
                                msg : 'Artifact bundle upload failed.',
                                buttons : Sonatype.MessageBox.OK,
                                icon : Sonatype.MessageBox.ERROR
                              });
                        }
                        else
                        {
                          var jsonResponse = Ext.decode(response.responseText);

                          var msg = 'Artifact Bundle upload finished successfully<br>';

                          if (jsonResponse.repositoryUris.length == 1)
                          {
                            msg += 'The following staging repository has been created<br><br>';
                          }
                          else if (jsonResponse.repositoryUris.length > 1)
                          {
                            msg += 'The following staging repositories have been created<br><br>';
                          }
                          for (var i = 0; i < jsonResponse.repositoryUris.length; i++)
                          {
                            msg += '<a href="' + jsonResponse.repositoryUris[i] + '" target="_new">' + jsonResponse.repositoryUris[i] + '</a>'
                          }

                          Sonatype.MessageBox.show({
                                title : 'Upload Complete',
                                msg : msg,
                                buttons : Sonatype.MessageBox.OK,
                                icon : Sonatype.MessageBox.INFO
                              });
                        }
                      }
                      else
                      {
                        var s = 'Artifact upload failed.<br />';
                        var r = response.responseText;
                        var n1 = r.toLowerCase().indexOf('<error>') + 7;
                        var n2 = r.toLowerCase().indexOf('</error>');
                        if (n2 > n1)
                        {

                          if (r.indexOf('failures') !== -1)
                          {
                            Sonatype.MessageBox.hide();
                            // there is markup like <b> in the rule messages that is 'double encoded' because of
                            // weirdo upload error handling
                            this.showRulesetEvaluationReport(Nexus.util.Format.htmlDecode(r.substring(n1, n2)));
                            s = null;
                          } else {
                            s += r.substring(n1, n2);
                          }
                        }
                        else
                        {
                          s += 'Check Nexus logs for more information.';
                        }

                        if (s)
                        {
                          Sonatype.MessageBox.show({
                                title : 'Upload Failed',
                                msg : s,
                                buttons : Sonatype.MessageBox.OK,
                                icon : Sonatype.MessageBox.ERROR
                              });
                        }
                      }
                    }
                  });
            }
          }
        });
    Nexus.staging.Rules.treat(Sonatype.repoServer.StagingUploadPanel);

    Sonatype.Events.addListener('nexusNavigationInit', function(nexusPanel) {
          var enable = Sonatype.lib.Permissions.checkPermission('nexus:stagingupload', Sonatype.lib.Permissions.CREATE) ||
                 Sonatype.lib.Permissions.checkPermission('nexus:stagingbundleupload', Sonatype.lib.Permissions.CREATE);
          nexusPanel.add({
                enabled : enable,
                sectionId : 'st-enterprise-staging',
                title : 'Staging Upload',
                tabId : 'staging-upload',
                tabCode : Sonatype.repoServer.StagingUploadPanel
              });
        });

});